<?php
namespace VAC\VisitorAnalytics\Controller;


use VAC\VisitorAnalytics\Service\ApiService;
use VAC\VisitorAnalytics\Domain\Repository\AnalyticsRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;
use TYPO3\CMS\Core\Page\PageRenderer;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;


/***
 *
 * This file is part of the "Visitor Analytics Core" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019
 *
 ***/

/**
 * AnalyticsController
 */
class AnalyticsController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
	protected $analyticsRepository;
	protected $apiServiceObject;

	public function __construct(AnalyticsRepository $analyticsRepository)
	{
		$this->analyticsRepository = $analyticsRepository;
		$this->apiServiceObject = GeneralUtility::makeInstance('VAC\\VisitorAnalytics\\Service\\ApiService');
	}

    /**
     * action list
     *
     * @param
     * @return void
     */

    public function listAction()
    {
        return true;
    }

    /**
     * action show
     *
     * @param \VAC\VisitorAnalytics\Domain\Model\Analytics $analytics
     * @return void
     */

    public function showAction(\VAC\VisitorAnalytics\Domain\Model\Analytics $analytics)
    {
        return true;
    }

    /**
     * action login
     *
     * @param void
     * @return void
     */

	public function loginAction()
	{
		$baseUrl = $this->request->getBaseUri();
		$baseUrl = str_replace("/typo3/",'',$baseUrl);
		$recievedData = $this->request->getArguments();
		if (!$this->request->hasArgument('registered')) {
			if (!isset($recievedData['tokenExpired']) && ($this->request->getMethod() != 'POST')) {
				$this->refreshToken();
			}
		} else {
			$this->view->assign('responseMessage', 'REGISTERD_NOW');
		}
		$status = $this->checkStatus($baseUrl);
		if ($this->request->getMethod() == 'POST') {
			$headers = [
				'ContentType' => 'application/json'
			];
			$data = [
				'email' => $_REQUEST['email'],
				'password' => $_REQUEST['password'],
			];
			$api = '/api/typo/token';
			$apiData = $this->apiServiceObject->doCurlPostRequest($api, $headers, json_encode($data));
			$response = json_decode($apiData);
			if (!empty($response)) {
				if ($response->error) {
					$this->view->assign('responseMessage', $response->error);
				} else {
                    $analyticsInfoAll = $this->analyticsRepository->findAll();
                    $analyticsInfo = $analyticsInfoAll->getFirst();

					if ($analyticsInfo) {
						$analyticsInfo->setAccessToken($response->token);
						$analyticsInfo->setRefreshToken($response->refreshToken);
						$analyticsInfo->setUserId($response->userId);
						$this->analyticsRepository->update($analyticsInfo);
						$this->getWebsite($baseUrl);
					} else {
						$analyticsInfo = GeneralUtility::makeInstance('VAC\\VisitorAnalytics\\Domain\\Model\\Analytics');
						$analyticsInfo->setAccessToken($response->token);
						$analyticsInfo->setRefreshToken($response->refreshToken);
						$analyticsInfo->setUserId($response->userId);
						$this->analyticsRepository->add($analyticsInfo);
						$this->getWebsite($baseUrl);
					}
				}
			}
		}
	}

    /**
     * action register
     *
     * @param void
     * @return void
     */

	public function registerAction()
	{
		if ($this->request->getMethod() == 'POST') {
			$data = $this->request->getArguments();
			$timeZone = $_REQUEST['timezone'];
			$headers = [
				'ContentType' => 'application/json'
			];
			if (!empty($data)) {
				$data = [
					'firstName' => $_REQUEST['firstName'],
					'lastName' => $_REQUEST['lastName'],
					'companyName' => $_REQUEST['companyName'],
					'email' => $_REQUEST['email'],
					'password' => $_REQUEST['password'],
					'privacyTermsAccepted' => ($_REQUEST['termsAndDpaAccepted'] == 'on') ? true : false,
					'dpaAccepted' => true,
					'businessConditionAccepted' => true,
					'newsletterSubscribed' => ($_REQUEST['newsAndOffers'] == 'on') ? true : false,
					'source' => 'TYPO'
				];
				$api = '/api/users';
				$apiData = $this->apiServiceObject->doCurlPostRequest($api, $headers, json_encode($data));
				$response = json_decode($apiData);
				if (!empty($response)) {
					if ($response->error) {
						$this->view->assign('responseMessage', $response->error);
					} else {
						$analyticsInfoAll = $this->analyticsRepository->findAll();
	                    $analyticsInfo = $analyticsInfoAll->getFirst();
						if ($analyticsInfo) {
							$analyticsInfo->setUserId($response->id);
							$analyticsInfo->setTimeZone($timeZone);
							$this->analyticsRepository->update($analyticsInfo);
						} else {
							$analyticsInfo = GeneralUtility::makeInstance('VAC\\VisitorAnalytics\\Domain\\Model\\Analytics');
							$analyticsInfo->setUserId($response->id);
							$analyticsInfo->setTimezone($timeZone);
							$this->analyticsRepository->add($analyticsInfo);
						}
						$this->redirect('login', null, null, ['registered' => true]);
					}
				}
			}
		}
	}

    /**
     * action addWebsite
     *
     * @param void
     * @return void
     */

	public function addWebsiteAction()
	{
		$data = $this->request->getArguments();
	    $analyticsInfoAll = $this->analyticsRepository->findAll();
	    $analyticsInfo = $analyticsInfoAll->getFirst();
		if ($analyticsInfo) {
			$token = $analyticsInfo->getAccessToken();
		} else {
			$this->redirect('login');
		}
		$headers[] = "Authorization:Bearer".' '.$token;
		if (!empty($data)) {
			$data = [
				'url' => $data['baseUrl'],
				'timeZone' => ($analyticsInfo->getTimezone()) ? $analyticsInfo->getTimezone() : 'Europe/London',
				'platform' => 'TYPO'
			];
			$api = '/api/websites';
			$apiData = $this->apiServiceObject->doCurlPostRequest($api, $headers, json_encode($data));
			$response = json_decode($apiData);
			if (!empty($response)) {
				if ($response->error) {
					if ($response->error == 'INVALID_DATA') {
						$responseMessage = 'Please fill details';
                        $severity = \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO;
					} else if ($response->error == 'NOT_FOUND_WEBSITE') {
						$responseMessage = 'Website not found';
                        $severity = \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING;
					} else if ($response->error == 'MONGO_WEBSITE_NOT_SYNCED') {
						$responseMessage = 'Website not synched';
                        $severity = \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING;
					} else if ($response->error == 'AUTHENTICATION_INVALID_JWT_TOKEN') {
						return $this->redirect('login');
					}
				} else {
                    $analyticsInfoAll = $this->analyticsRepository->findAll();
                    $analyticsInfo = $analyticsInfoAll->getFirst();
					$analyticsInfo->setWebsiteId($response->id);
					$analyticsInfo->setSiteUrl($response->url);
					$this->analyticsRepository->update($analyticsInfo);
					$this->activateTrial();
				}
			}
		}
	}

    /**
     * function refreshToken
     *
     * @param void
     * @return void
     */

	public function refreshToken()
	{
        $analyticsInfoAll = $this->analyticsRepository->findAll();
        $analyticsInfo = $analyticsInfoAll->getFirst();
		if ($analyticsInfo && $analyticsInfo->getRefreshToken()) {
			$api = '/api/typo/token-refresh';
			$headers = [
				'Content-Type' => 'application/json'
			];
			$data = [
				'refreshToken' => $analyticsInfo->getRefreshToken()
			];
			$apiData = $this->apiServiceObject->doCurlPostRequest($api, $headers, json_encode($data));
			if ($apiData) {
				$response = json_decode($apiData);
				if (!empty($response)) {
					if ($response->error) {
						return $this->redirect('login', null, null, ['tokenExpired' => true]);
					} else {
                        $analyticsInfo->setAccessToken($response->token);
                        $analyticsInfo->setRefreshToken($response->refreshToken);
                        $this->analyticsRepository->update($analyticsInfo);

						return true;
                    }
				}
			}
		}
		return $this->redirect('login', null, null, ['tokenExpired' => true]);
	}

    /**
     * function checkStatus
     *
     * @param void
     * @return void
     */

	public function checkStatus($baseUrl)
	{
        $analyticsInfoAll = $this->analyticsRepository->findAll();
        $analyticsInfo = $analyticsInfoAll->getFirst();
		$status = 0;
		if ($analyticsInfo && $analyticsInfo->getAccessToken() && !$analyticsInfo->getWebsiteId()) {
            $this->getWebsite($baseUrl); // Check whether website exist or active
		    //$this->redirect('addWebsite', null, null, ['baseUrl' => $baseUrl]); // Not added website
		}
		if ($analyticsInfo && $analyticsInfo->getAccessToken() && $analyticsInfo->getWebsiteId() && !$analyticsInfo->getTrackingCode()) {
			$this->getWebsite($baseUrl); // Check whether website exist or active
		}
		if ($analyticsInfo && $analyticsInfo->getAccessToken() && $analyticsInfo->getWebsiteId() && $analyticsInfo->getTrackingCode()) {
		   	// To check whether token has expired or not. If not analytics will be shown
			$this->getWebsite($baseUrl);
			//$this->redirect('showAnalytics');
		}
		return $status;
	}

    /**
     * action showAnalytics
     *
     * @param void
     * @return void
     */

	public function showAnalyticsAction()
	{
        $objectManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Object\\ObjectManager');
		$tokenStatus = $this->refreshToken();
		if ($tokenStatus) {
            $analyticsInfoAll = $this->analyticsRepository->findAll();
            $analyticsInfo = $analyticsInfoAll->getFirst();
			$this->view->assignMultiple([
				'userId' => $analyticsInfo->getUserId(),
				'wid' => $analyticsInfo->getWebsiteId(),
				'token' => $analyticsInfo->getAccessToken()
			]);
		}
	}

    /**
     * action generateCode
     *
     * @param void
     * @return void
     */

	public function generateCodeAction()
	{
        $analyticsInfoAll = $this->analyticsRepository->findAll();
        $analyticsInfo = $analyticsInfoAll->getFirst();
		if ($analyticsInfo) {
			$websiteId = $analyticsInfo->getWebsiteId();
			$token = $analyticsInfo->getAccessToken();
			$headers[] = "Authorization:Bearer".' '.$token;
			$api = '/api/websites/'.$websiteId.'/tracking-code';
			$apiData = $this->apiServiceObject->doCurlGetRequest($api, $headers);
			$response = json_decode($apiData);
			if ($response->error) {
				if ($response->error == 'AUTHENTICATION_INVALID_JWT_TOKEN' || $response->error == 'AUTHENTICATION_USER_NOT_FOUND') {
					$this->redirect('login');
				}
			} else {
		        $analyticsInfoAll = $this->analyticsRepository->findAll();
		        $analyticsInfo = $analyticsInfoAll->getFirst();
				$analyticsInfo->setTrackingCode($response->trackingCode);
				$this->analyticsRepository->update($analyticsInfo);
				$this->redirect('showAnalytics');
			}
		} else {
				$this->redirect('login');
			}
	}

    /**
     * function .
     *
     * @param void
     * @return void
     */

	public function getWebsite($baseUrl)
	{
        $analyticsInfoAll = $this->analyticsRepository->findAll();
        $analyticsInfo = $analyticsInfoAll->getFirst();
		if (!empty($analyticsInfo)) {
			$token = $analyticsInfo->getAccessToken();
		} else {
			$this->redirect('login');
		}
		$headers[] = "Authorization:Bearer".' '.$token;
		$api = '/api/websites?url='.$baseUrl;
		$apiData = $this->apiServiceObject->doCurlGetRequest($api, $headers);
		$response = json_decode($apiData);
		if ($response->error) {
			if ($response->error == 'NOT_FOUND_WEBSITE') {
				$this->redirect('addWebsite', null, null, ['baseUrl' => $baseUrl]);
			}
		} else {
            $analyticsInfoAll = $this->analyticsRepository->findAll();
            $analyticsInfo = $analyticsInfoAll->getFirst();
			$analyticsInfo->setWebsiteId($response->id);
			$analyticsInfo->setSiteUrl($response->url);
			$this->analyticsRepository->update($analyticsInfo);
			$this->redirect('generateCode');
		}
	}

    /**
     * function activateTrial
     *
     * @param void
     * @return void
     */

	public function activateTrial()
	{
        $analyticsInfoAll = $this->analyticsRepository->findAll();
        $analyticsInfo = $analyticsInfoAll->getFirst();
		if ($analyticsInfo) {
			$token = $analyticsInfo->getAccessToken();
			$websiteId = $analyticsInfo->getWebsiteId();
			$headers[] = "Authorization:Bearer".' '.$token;
			$api = '/api/websites/'.$websiteId.'/subscriptions/free-trial';
			$apiData = $this->apiServiceObject->doCurlPostRequest($api, $headers, $data = '');
			$response = json_decode($apiData);
			if ($response) {
					$this->redirect('generateCode');
			}
		}
	}

    /**
     * action activateTrackingCode
     *
     * @param void
     * @return void
     */

    public function activateTrackingCodeAction()
    {
        $pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
        $analyticsInfoAll = $this->analyticsRepository->findAll();
        $analyticsInfo = $analyticsInfoAll->getFirst();
        if (!empty($analyticsInfo)) {
            $trackingCode = $analyticsInfo->getTrackingCode();
            if(!empty($trackingCode)){
                $pageRenderer->addHeaderData($trackingCode);
            }
        }
    }
}
